<?php

namespace EventPortal\Http\Controllers\Admin\Email;

use EventPortal\Domain\Email\Models\EventEmail;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\AdminRender;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Response;

class EmailController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:email.show')->only('index');
        $this->middleware('check-permission:email.update')->only(['edit', 'update']);
    }

    public function index(Event $event): Response
    {
        $table = Table::make('email.index')
            ->context($event)
            ->query(EventEmail::forEvent($event))
            ->request(request())
            ->get();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event->id))
            ->add('Emails')
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->render();

        return AdminRender::component('Admin/Email/Index')
            ->key('event.email.index')
            ->withTable($table)
            ->withBreadcrumbs($breadcrumbs)
            ->withSideBar($sidebar)
            ->get();
    }

    public function show(Event $event, EventEmail $email): Response
    {
        $form = Form::make('email.show')
            ->model($email)
            ->context($event)
            ->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event->id))
            ->add('Emails', route('admin.email.index', [
                'event' => $event->id,
            ]))
            ->add($email->type)
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->user(auth()->user())
            ->model($event)
            ->render();

        return AdminRender::component('Admin/Email/Show')
            ->key('event.email.show')
            ->withData([
                'data' => $form,
                'route' => 'admin.email.preview.show',
                'routeParams' => [
                    'event' => $event->id,
                    'email' => $email->id,
                ],
            ])
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($email->type)
            ->withSideBar($sidebar)
            ->get();
    }

    public function edit(Event $event, EventEmail $email): Response
    {
        $form = Form::make('email.edit')->model($email)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event->id))
            ->add('Emails', route('admin.email.index', [
                'event' => $event->id,
            ]))
            ->add($email->type, route('admin.email.show', [
                'event' => $event->id,
                'email' => $email->id,
            ]))
            ->add("Edit {$email->type} email")
            ->get();

        return AdminRender::component('Admin/Email/Edit')
            ->key('event.email.edit')
            ->withForm($form)
            ->withData([
                'route' => 'admin.email.preview.show',
                'routeParams' => [
                    'event' => $event->id,
                    'email' => $email->id,
                ],
            ])
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle("Edit {$email->type} email")
            ->get();
    }

    public function update(Request $request, Event $event, EventEmail $email): RedirectResponse
    {
        Form::make('email.edit')
            ->model($email)
            ->validate($request->all())
            ->handle($request->all());

        toast()->success(trans('email.updated'));

        return redirect()->route('admin.email.show', [
            'event' => $event,
            'email' => $email,
        ]);
    }
}
